/* linux/arch/arm/mach-s3c24a0/include/mach/regs-irq.h
 *
 * Copyright (c) 2003 Simtec Electronics <linux@simtec.co.uk>
 *		      http://www.simtec.co.uk/products/SWLINUX/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/


#ifndef ___ASM_ARCH_24A0_REGS_IRQ_H
#define ___ASM_ARCH_24A0_REGS_IRQ_H __FILE__


#define S3C2410_EINTMASK	S3C2410_EINTREG(0x034)
#define S3C2410_EINTPEND	S3C2410_EINTREG(0X038)

#define S3C24XX_EINTMASK	S3C24XX_EINTREG(0x034)
#define S3C24XX_EINTPEND	S3C24XX_EINTREG(0X038)

#endif /* __ASM_ARCH_24A0_REGS_IRQ_H */



